@@ -1,4 +1,4 @@
-/*	$Id: document.c,v 1.166 2021/02/01 14:58:24 kristaps Exp $ */
+/*	$Id: document.c,v 1.172 2021/02/06 10:42:25 kristaps Exp $ */
 /*
  * Copyright (c) 2008, Natacha Porté
  * Copyright (c) 2011, Vicent Martí
@@ -84,7 +84,6 @@ struct footnote_ref {
 TAILQ_HEAD(footnote_refq, footnote_ref);
 
 struct 	lowdown_doc {
-	const struct lowdown_opts *opts;
 	struct link_refq refq; /* all internal references */
 	struct footnote_refq footnotes; /* all footnotes */
 	size_t		 footnotesz; /* # of used footnotes */
@@ -97,6 +96,10 @@ struct 	lowdown_doc {
 	struct hbufq	 metaq; /* raw metadata key/values */
 	size_t		 depth; /* current parse tree depth */
 	size_t		 maxdepth; /* max parse tree depth */
+	char		**meta; /* primer metadata */
+	size_t		  metasz; /* key-value pairs in meta */
+	char		**metaovr; /* override metadata */
+	size_t		  metaovrsz; /* key-value pairs in metaovr */
 };
 
 /*
@@ -1275,7 +1278,7 @@ char_autolink_www(struct lowdown_doc *doc,
 {
 	struct lowdown_buf	*link = NULL, *link_url = NULL;
 	size_t	 		 link_len, rewind;
-	struct lowdown_node 	*n, *nn;
+	struct lowdown_node 	*n;
 	ssize_t			 ret;
 
 	if (doc->in_link_body)
@@ -1305,17 +1308,12 @@ char_autolink_www(struct lowdown_doc *doc,
 				n->rndr_normal_text.text.size = 0;
 		}
 
-		if ((n = pushnode(doc, LOWDOWN_LINK)) == NULL)
+		if ((n = pushnode(doc, LOWDOWN_LINK_AUTO)) == NULL)
 			goto err;
-		if (!pushbuffer(&n->rndr_link.link, 
+		n->rndr_autolink.type = HALINK_NORMAL;
+		if (!pushbuffer(&n->rndr_autolink.link, 
 		    link_url->data, link_url->size))
 			goto err;
-		if ((nn = pushnode(doc, LOWDOWN_NORMAL_TEXT)) == NULL)
-			goto err;
-		if (!pushbuffer(&nn->rndr_normal_text.text, 
-		    link->data, link->size))
-			goto err;
-		popnode(doc, nn);
 		popnode(doc, n);
 	}
 
@@ -4001,6 +3999,7 @@ lowdown_doc_new(const struct lowdown_opts *opts)
 {
 	struct lowdown_doc	*doc;
 	unsigned int		 extensions = opts ? opts->feat : 0;
+	size_t			 i;
 
 	doc = calloc(1, sizeof(struct lowdown_doc));
 	if (doc == NULL)
@@ -4030,10 +4029,35 @@ lowdown_doc_new(const struct lowdown_opts *opts)
 	if (extensions & LOWDOWN_MATH)
 		doc->active_char['$'] = MD_CHAR_MATH;
 
-	doc->opts = opts;
 	doc->ext_flags = extensions;
 
+	if (opts->metasz > 0) {
+		doc->meta = calloc(opts->metasz, sizeof(char *));
+		if (doc->meta == NULL)
+			goto err;
+		doc->metasz = opts->metasz;
+		for (i = 0; i < doc->metasz; i++) {
+			doc->meta[i] = strdup(opts->meta[i]);
+			if (doc->meta[i] == NULL)
+				goto err;
+		}
+	}
+	if (opts->metaovrsz > 0) {
+		doc->metaovr = calloc(opts->metaovrsz, sizeof(char *));
+		if (doc->metaovr == NULL)
+			goto err;
+		doc->metaovrsz = opts->metaovrsz;
+		for (i = 0; i < doc->metaovrsz; i++) {
+			doc->metaovr[i] = strdup(opts->metaovr[i]);
+			if (doc->metaovr[i] == NULL)
+				goto err;
+		}
+	}
+
 	return doc;
+err:
+	lowdown_doc_free(doc);
+	return NULL;
 }
 
 /*
@@ -4136,11 +4160,11 @@ parse_metadata_val(const char *data, size_t sz, size_t *len)
 static int
 parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 {
-	size_t	 	 	 i, j, pos = 0, valsz, keysz;
-	const char		*key, *val;
+	size_t	 	 	 i, j, pos = 0, vsz, keysz;
 	struct hbufn		*m;
 	struct lowdown_node	*n, *nn;
-	char			*cp;
+	const char		*val, *key;
+	char			*cp, *buf;
 
 	if (sz == 0 || data[sz - 1] != '\n')
 		return 0;
@@ -4165,20 +4189,14 @@ parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 	 */
 
 	for (pos = 0; pos < sz; ) {
-		if ((m = calloc(1, sizeof(struct hbufn))) == NULL)
-			return -1;
-		TAILQ_INSERT_TAIL(&doc->metaq, m, entries);
-
-		if ((n = pushnode(doc, LOWDOWN_META)) == NULL)
-			return -1;
-
-		m->key = &n->rndr_meta.key;
-
 		key = &data[pos];
 		for (i = pos; i < sz; i++)
 			if (data[i] == ':')
 				break;
+
 		keysz = i - pos;
+		if ((cp = buf = malloc(keysz + 1)) == NULL)
+			return -1;
 
 		/*
 		 * Normalise the key to lowercase alphanumerics, "-",
@@ -4186,10 +4204,6 @@ parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 		 * with a question mark.
 		 */
 
-		n->rndr_meta.key.data = cp = malloc(keysz);
-		if (cp == NULL)
-			return -1;
-
 		for (j = 0; j < keysz; j++) {
 			if (isalnum((unsigned char)key[j]) ||
 			    '-' == key[j] || '_' == key[j]) {
@@ -4199,15 +4213,45 @@ parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 				continue;
 			*cp++ = '?';
 		}
-		n->rndr_meta.key.size = cp - n->rndr_meta.key.data;
+		*cp = '\0';
 
-		/* Canonical order: title comes first. */
+		/* 
+		 * If we've already encountered this key, remove it from
+		 * both the local queue and the meta nodes.
+		 */
 
-		if (hbuf_streq(&n->rndr_meta.key, "title")) {
-			TAILQ_REMOVE(&n->parent->children, n, entries);
-			TAILQ_INSERT_HEAD(&n->parent->children, n, entries);
+		TAILQ_FOREACH(m, &doc->metaq, entries)
+			if (hbuf_streq(m->key, buf)) {
+				TAILQ_REMOVE(&doc->metaq, m, entries);
+				free(m);
+				break;
+			}
+
+		assert(doc->current->type == LOWDOWN_DOC_HEADER);
+		TAILQ_FOREACH(n, &doc->current->children, entries) {
+			assert(n->type == LOWDOWN_META);
+			if (hbuf_streq(&n->rndr_meta.key, buf)) {
+				TAILQ_REMOVE(&doc->current->children, n, entries);
+				lowdown_node_free(n);
+				break;
+			}
 		}
 
+		if ((n = pushnode(doc, LOWDOWN_META)) == NULL) {
+			free(buf);
+			return -1;
+		}
+		if (!pushbuffer(&n->rndr_meta.key, buf, cp - buf)) {
+			free(buf);
+			return -1;
+		}
+		free(buf);
+
+		if ((m = calloc(1, sizeof(struct hbufn))) == NULL)
+			return -1;
+		TAILQ_INSERT_TAIL(&doc->metaq, m, entries);
+		m->key = &n->rndr_meta.key;
+
 		if (i == sz) {
 			popnode(doc, n);
 			break;
@@ -4224,15 +4268,16 @@ parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 			break;
 		}
 
-		val = parse_metadata_val(&data[i], sz - i, &valsz);
+		val = parse_metadata_val(&data[i], sz - i, &vsz);
 		if ((nn = pushnode(doc, LOWDOWN_NORMAL_TEXT)) == NULL)
 			return -1;
 		m->val = &nn->rndr_normal_text.text;
-		if (!pushbuffer(&nn->rndr_normal_text.text, val, valsz))
+		if (!pushbuffer(&nn->rndr_normal_text.text, val, vsz))
 			return -1;
 		popnode(doc, nn);
 		popnode(doc, n);
-		pos = i + valsz + 1;
+
+		pos = i + vsz + 1;
 	}
 
 	return 1;
@@ -4250,7 +4295,7 @@ lowdown_doc_parse(struct lowdown_doc *doc,
 {
 	static const char 	 UTF8_BOM[] = { 0xEF, 0xBB, 0xBF };
 	struct lowdown_buf	*text;
-	size_t		 	 beg, end;
+	size_t		 	 beg, end, i;
 	const char		*sv;
 	struct lowdown_node 	*n, *root = NULL;
 	struct hbufn		*m;
@@ -4283,14 +4328,21 @@ lowdown_doc_parse(struct lowdown_doc *doc,
 	if (size >= 3 && memcmp(data, UTF8_BOM, 3) == 0)
 		beg += 3;
 
+	/*
+	 * Zeroth pass: metadata.  First process given metadata, then
+	 * in-document metadata, then overriding metadata.  The
+	 * in-document metadata is conditionally processed.
+	 */
+
 	if ((n = pushnode(doc, LOWDOWN_DOC_HEADER)) == NULL)
 		goto out;
 
-	/*
-	 * Zeroth pass: see if we should collect metadata.
-	 * Only do so if we're toggled to look for metadata.
-	 * (Only parse if we must.)
-	 */
+	for (i = 0; i < doc->metasz; i++)
+		if (parse_metadata(doc, 
+		    doc->meta[i], strlen(doc->meta[i])) < 0)
+			goto out;
+
+	/* FIXME: CRLF EOLNs. */
 
 	if ((doc->ext_flags & LOWDOWN_METADATA) &&
 	    beg < size - 1 && 
@@ -4307,6 +4359,13 @@ lowdown_doc_parse(struct lowdown_doc *doc,
 			goto out;
 	}
 
+	for (i = 0; i < doc->metaovrsz; i++)
+		if (parse_metadata(doc, 
+		    doc->metaovr[i], strlen(doc->metaovr[i])) < 0)
+			goto out;
+
+	popnode(doc, n);
+
 	/* 
 	 * First pass: looking for references and footnotes, copying
 	 * everything else. 
@@ -4355,8 +4414,6 @@ lowdown_doc_parse(struct lowdown_doc *doc,
 		beg = end;
 	}
 
-	popnode(doc, n);
-
 	/* 
 	 * Second pass (after header): rendering the document body and
 	 * footnotes.
@@ -4491,6 +4548,17 @@ lowdown_metaq_free(struct lowdown_metaq *q)
 void
 lowdown_doc_free(struct lowdown_doc *doc)
 {
+	size_t	 i;
 
+	if (doc == NULL)
+		return;
+
+	for (i = 0; i < doc->metasz; i++)
+		free(doc->meta[i]);
+	for (i = 0; i < doc->metaovrsz; i++)
+		free(doc->metaovr[i]);
+
+	free(doc->meta);
+	free(doc->metaovr);
 	free(doc);
 }
