@@ -1,4 +1,4 @@
-/*	$Id: nroff.c,v 1.143 2021/02/01 13:11:54 kristaps Exp $ */
+/*	$Id: nroff.c,v 1.148 2021/02/08 08:48:27 kristaps Exp $ */
 /*
  * Copyright (c) 2008, Natacha Porté
  * Copyright (c) 2011, Vicent Martí
@@ -70,6 +70,7 @@ struct	bnode {
 	char				*nargs; /* (safe) 1st args */
 	char				*args; /* (unsafe) 2nd args */
 	int				 close; /* BNODE_COLOUR/FONT */
+	int				 tblhack; /* BSCOPE_SPAN */
 	enum bscope			 scope; /* scope */
 	unsigned int			 font; /* if BNODE_FONT */
 #define	BFONT_ITALIC			 0x01
@@ -149,6 +150,7 @@ hesc_nroff(struct lowdown_buf *ob, const char *data,
 			if (!HBUF_PUTSL(ob, "\\e"))
 				return 0;
 			break;
+		case '\'':
 		case '.':
 			if (!oneline &&
 			    ob->size > 0 && 
@@ -374,6 +376,18 @@ bqueue_flush(struct lowdown_buf *ob, const struct bnodeq *bq)
 				return 0;
 		}
 
+		/* 
+		 * A "tblhack" is used by a span macro to indicate
+		 * that it should start its own line, but that data
+		 * continues to flow after it.  This is only used in
+		 * tables with T}, at this point.
+		 */
+
+		if (bn->scope == BSCOPE_SPAN && bn->tblhack &&
+		    ob->size > 0 && ob->data[ob->size - 1] != '\n')
+			if (!hbuf_putc(ob, '\n'))
+				return 0;
+
 		/* Safe data need not be escaped. */
 
 		if (bn->nbuf != NULL && !hbuf_puts(ob, bn->nbuf))
@@ -452,6 +466,33 @@ out:
 	return rc;
 }
 
+/*
+ * Convert a link into a short-link and place the escaped output into a
+ * returned string.
+ * Returns NULL on memory allocation failure.
+ */
+static char *
+hbuf2shortlink(const struct lowdown_buf *link)
+{
+	struct lowdown_buf	*tmp = NULL, *slink = NULL;
+	char			*ret = NULL;
+
+	if ((tmp = hbuf_new(32)) == NULL)
+		goto out;
+	if ((slink = hbuf_new(32)) == NULL)
+		goto out;
+	if (!hbuf_shortlink(tmp, link))
+		goto out;
+	if (!hesc_nroff(slink, 
+	    tmp->data, tmp->size, 1, 0))
+		goto out;
+	ret = strndup(slink->data, slink->size);
+out:
+	hbuf_free(tmp);
+	hbuf_free(slink);
+	return ret;
+}
+
 /*
  * Manage hypertext linking with the groff "pdfhref" macro or simply
  * using italics.
@@ -474,30 +515,74 @@ putlink(struct bnodeq *obq, struct nroff *st,
 	int				 rc = 0;
 	ssize_t				 ret = 0;
 
+	/*
+	 * For -Tman or without .pdfhref, format the link as-is, with
+	 * text then link, or use the various shorteners.
+	 */
+
 	if (st->man || !(st->flags & LOWDOWN_NROFF_GROFF)) {
+		if (bq == NULL) {
+			st->fonts[NFONT_ITALIC]++;
+			if (!bqueue_font(st, obq, 0))
+				goto out;
+			if ((bn = bqueue_span(obq, NULL)) == NULL)
+				goto out;
+			if (st->flags & LOWDOWN_NROFF_SHORTLINK) {
+				bn->nbuf = hbuf2shortlink(link);
+				if (bn->nbuf == NULL)
+					goto out;
+			} else
+				bn->buf = link;
+			st->fonts[NFONT_ITALIC]--;
+			if (!bqueue_font(st, obq, 1))
+				goto out;
+			rc = 1;
+			goto out;
+		}
+		st->fonts[NFONT_BOLD]++;
+		if (!bqueue_font(st, obq, 0))
+			goto out;
+		TAILQ_CONCAT(obq, bq, entries);
+		st->fonts[NFONT_BOLD]--;
+		if (!bqueue_font(st, obq, 1))
+			goto out;
+		if (st->flags & LOWDOWN_NROFF_NOLINK) {
+			rc = 1;
+			goto out;
+		}
+		if (bqueue_span(obq, " (") == NULL)
+			goto out;
 		st->fonts[NFONT_ITALIC]++;
 		if (!bqueue_font(st, obq, 0))
-			return -1;
-		if (bq == NULL) {
-			if ((bn = bqueue_span(obq, NULL)) == NULL)
-				return -1;
-			bn->buf = link;
+			goto out;
+		if ((bn = bqueue_span(obq, NULL)) == NULL)
+			goto out;
+		if (st->flags & LOWDOWN_NROFF_SHORTLINK) {
+			bn->nbuf = hbuf2shortlink(link);
+			if (bn->nbuf == NULL)
+				goto out;
 		} else
-			TAILQ_CONCAT(obq, bq, entries);
+ 			bn->buf = link;
 		st->fonts[NFONT_ITALIC]--;
-		return bqueue_font(st, obq, 1) ? 0 : -1;
+		if (!bqueue_font(st, obq, 1))
+			goto out;
+		if (bqueue_span(obq, ")") == NULL)
+			goto out;
+		rc = 1;
+		goto out;
 	}
 
+	/* Otherwise, use .pdfhref. */
+
 	if ((ob = hbuf_new(32)) == NULL)
 		goto out;
 
-	prev = TAILQ_LAST(obq, bnodeq);
-
 	/*
 	 * If we're preceded by normal text that doesn't end with space,
 	 * then put that text into the "-P" (prefix) argument.
 	 */
 
+	prev = TAILQ_LAST(obq, bnodeq);
 	if (prev != NULL &&
 	    prev->scope == BSCOPE_SPAN &&
 	    prev->buf != NULL &&
@@ -528,6 +613,12 @@ putlink(struct bnodeq *obq, struct nroff *st,
 		prev->buf = NULL;
 	}
 
+	/* 
+	 * If we're followed by text flush against the node, "chop" the
+	 * node until the next white-space.  This will be handed to the
+	 * next rndr() invocation.
+	 */
+
 	if (next != NULL && 
 	    next->type == LOWDOWN_NORMAL_TEXT) {
 		nbuf = &next->rndr_normal_text.text;
@@ -883,15 +974,45 @@ rndr_hrule(const struct nroff *st, struct bnodeq *obq)
 }
 
 static int
-rndr_image(const struct nroff *st, struct bnodeq *obq, 
+rndr_image(struct nroff *st, struct bnodeq *obq, 
 	const struct rndr_image *param)
 {
 	const char	*cp;
 	size_t		 sz;
 	struct bnode	*bn;
 
-	if (st->man)
-		return 1;
+	/* In -Tman, we have no images: treat as a link. */
+
+	if (st->man) {
+		st->fonts[NFONT_BOLD]++;
+		if (!bqueue_font(st, obq, 0))
+			return 0;
+		if ((bn = bqueue_span(obq, NULL)) == NULL)
+			return 0;
+		bn->buf = &param->alt;
+		st->fonts[NFONT_BOLD]--;
+		if (!bqueue_font(st, obq, 1))
+			return 0;
+		if (st->flags & LOWDOWN_NROFF_NOLINK)
+			return bqueue_span(obq, " (Image)") != NULL;
+		if (bqueue_span(obq, " (Image: ") == NULL)
+			return 0;
+		st->fonts[NFONT_ITALIC]++;
+		if (!bqueue_font(st, obq, 0))
+			return 0;
+		if ((bn = bqueue_span(obq, NULL)) == NULL)
+			return 0;
+		if (st->flags & LOWDOWN_NROFF_SHORTLINK) {
+			bn->nbuf = hbuf2shortlink(&param->link);
+			if (bn->nbuf == NULL)
+				return 0;
+		} else
+			bn->buf = &param->link;
+		st->fonts[NFONT_ITALIC]--;
+		if (!bqueue_font(st, obq, 1))
+			return 0;
+		return bqueue_span(obq, ")") != NULL;
+	}
 
 	/* Are we suffixed with ps or eps? */
 
@@ -1030,13 +1151,17 @@ static int
 rndr_table_cell(struct bnodeq *obq, struct bnodeq *bq,
 	const struct rndr_table_cell *param)
 {
+	struct bnode	*bn;
 
 	if (param->col > 0 && bqueue_span(obq, "|") == NULL)
 		return 0;
-	if (bqueue_block(obq, "T{"))
+	if (bqueue_span(obq, "T{\n") == NULL)
 		return 0;
 	TAILQ_CONCAT(obq, bq, entries);
-	return bqueue_block(obq, "T}") != NULL;
+	if ((bn = bqueue_span(obq, "T}")) == NULL)
+		return 0;
+	bn->tblhack = 1;
+	return 1;
 }
 
 static int
