@@ -1,4 +1,4 @@
-/*	$Id: term.c,v 1.44 2021/02/01 21:41:23 kristaps Exp $ */
+/*	$Id: term.c,v 1.45 2021/02/03 18:07:25 kristaps Exp $ */
 /*
  * Copyright (c) 2020--2021 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -237,76 +237,6 @@ rndr_escape(struct term *term, struct lowdown_buf *out,
 	return cols;
 }
 
-/*
- * Link shortener.
- * This only shows the domain name and last path/filename.
- * It uses the following algorithm:
- *   (1) strip schema (if none, print in full)
- *   (2) print domain following
- *   (3) if no path, return
- *   (4) if path, look for final path component
- *   (5) print final path component with /.../ if shortened
- * Return zero on failure (memory), non-zero on success.
- */
-static int
-rndr_short_link(struct lowdown_buf *out,
-	const struct lowdown_buf *link)
-{
-	size_t		 start = 0, sz;
-	const char	*cp, *rcp;
-
-	/* 
-	 * Skip the leading protocol.
-	 * If we don't find a protocol, leave it be.
-	 */
-
-	if (link->size > 7 && strncmp(link->data, "http://", 7) == 0)
-		start = 7;
-	else if (link->size > 8 && strncmp(link->data, "https://", 8) == 0)
-		start = 8;
-	else if (link->size > 7 && strncmp(link->data, "file://", 7) == 0)
-		start = 7;
-	else if (link->size > 7 && strncmp(link->data, "mailto:", 7) == 0)
-		start = 7;
-	else if (link->size > 6 && strncmp(link->data, "ftp://", 6) == 0)
-		start = 6;
-
-	if (start == 0)
-		return hbuf_putb(out, link);
-
-	sz = link->size;
-	if (link->data[link->size - 1] == '/')
-		sz--;
-
-	/* 
-	 * Look for the end of the domain name. 
-	 * If we don't have an end, then print the whole thing.
-	 */
-
-	cp = memchr(link->data + start, '/', sz - start);
-	if (cp == NULL)
-		return hbuf_put(out, link->data + start, sz - start);
-
-	if (!hbuf_put(out, 
-	    link->data + start, cp - (link->data + start)))
-		return 0;
-
-	/* 
-	 * Look for the filename.
-	 * If it's the same as the end of the domain, then print the
-	 * whole thing.
-	 * Otherwise, use a "..." between.
-	 */
-
-	rcp = memrchr(link->data + start, '/', sz - start);
-
-	if (rcp == cp)
-		return hbuf_put(out, cp, sz - (cp - link->data));
-
-	return HBUF_PUTSL(out, "/...") &&
-		hbuf_put(out, rcp, sz - (rcp - link->data));
-}
-
 /*
  * Output style "s" into "out" as an ANSI escape.
  * If "s" does not have any style information, output nothing.
@@ -1331,9 +1261,9 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 		rc = rndr_buf(p, ob, n, &n->rndr_codespan.text, NULL);
 		break;
 	case LOWDOWN_LINK_AUTO:
-		if ((p->opts & LOWDOWN_TERM_SHORTLINK)) {
+		if (p->opts & LOWDOWN_TERM_SHORTLINK) {
 			hbuf_truncate(p->tmp);
-			if (!rndr_short_link
+			if (!hbuf_shortlink
 			    (p->tmp, &n->rndr_autolink.link))
 				return 0;
 			rc = rndr_buf(p, ob, n, p->tmp, NULL);
@@ -1341,14 +1271,16 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 			rc = rndr_buf(p, ob, n, &n->rndr_autolink.link, NULL);
 		break;
 	case LOWDOWN_LINK:
+		if (p->opts & LOWDOWN_TERM_NOLINK)
+			break;
 		hbuf_truncate(p->tmp);
 		if (!HBUF_PUTSL(p->tmp, " "))
 			return 0;
 		if (!rndr_buf(p, ob, n, p->tmp, NULL))
 			return 0;
-		if ((p->opts & LOWDOWN_TERM_SHORTLINK)) {
+		if (p->opts & LOWDOWN_TERM_SHORTLINK) {
 			hbuf_truncate(p->tmp);
-			if (!rndr_short_link
+			if (!hbuf_shortlink
 			    (p->tmp, &n->rndr_link.link))
 				return 0;
 			rc = rndr_buf(p, ob, n, p->tmp, NULL);
@@ -1365,14 +1297,21 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 			if (!rndr_buf(p, ob, n, p->tmp, NULL))
 				return 0;
 		}
+		if (p->opts & LOWDOWN_TERM_NOLINK) {
+			hbuf_truncate(p->tmp);
+			if (!HBUF_PUTSL(p->tmp, "[Image]"))
+				return 0;
+			rc = rndr_buf(p, ob, n, p->tmp, &sty_imgurlbox);
+			break;
+		}
 		hbuf_truncate(p->tmp);
 		if (!HBUF_PUTSL(p->tmp, "[Image: "))
 			return 0;
 		if (!rndr_buf(p, ob, n, p->tmp, &sty_imgurlbox))
 			return 0;
-		if ((p->opts & LOWDOWN_TERM_SHORTLINK)) {
+		if (p->opts & LOWDOWN_TERM_SHORTLINK) {
 			hbuf_truncate(p->tmp);
-			if (!rndr_short_link
+			if (!hbuf_shortlink
 			    (p->tmp, &n->rndr_image.link))
 				return 0;
 			if (!rndr_buf(p, ob, n, p->tmp, &sty_imgurl))
